import 'package:apple_sign_in/apple_sign_in.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_facebook_login/flutter_facebook_login.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:ocg_app/ocg_app.dart';

class SocialLoginHandler {
  static Future<AppResponse<OAuthCredential>> signInWithGoogle() async {
    try {
      GoogleSignIn _googleSign = GoogleSignIn();
      await _googleSign.signOut();
      var response = await _googleSign.signIn();
      var authentication = await response.authentication;
      var credential = GoogleAuthProvider.credential(
          accessToken: authentication.accessToken,
          idToken: authentication.idToken);
      return AppResponse.named(data: credential);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Some error occurred");
    }
  }

  static Future<AppResponse<OAuthCredential>> signInWithFacebook() async {
    try {
      FacebookLogin _facebookLogin = FacebookLogin();
      await _facebookLogin.logOut();
      var authentication = await _facebookLogin.logIn([]);
      switch (authentication.status) {
        case FacebookLoginStatus.cancelledByUser:
          return AppResponse.named(error: "Cancelled by user");
          break;
        case FacebookLoginStatus.loggedIn:
          FacebookAuthCredential credential = FacebookAuthProvider.credential(
            authentication.accessToken.token,
          );
          return AppResponse.named(data: credential);
          break;
        default:
          return AppResponse.named(error: "Some error occurred");
      }
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  static Future<AppResponse<OAuthCredential>> signInWithApple() async {
    try {
      if (!await AppleSignIn.isAvailable()) {
        return AppResponse.named(error: "Not available");
      }
      var result = await AppleSignIn.performRequests(
          [AppleIdRequest(requestedScopes: [])]);
      switch (result.status) {
        case AuthorizationStatus.authorized:
          final appleIdCredential = result.credential;
          final oAuthProvider = OAuthProvider('apple.com');
          final credential = oAuthProvider.credential(
            idToken: String.fromCharCodes(appleIdCredential.identityToken),
            accessToken:
                String.fromCharCodes(appleIdCredential.authorizationCode),
          );
          return AppResponse.named(data: credential);
          break;
        case AuthorizationStatus.error:
          return AppResponse.named(
            error: result.error.toString(),
          );
        case AuthorizationStatus.cancelled:
          return AppResponse.named(
            error: "Cancelled by user",
          );
        default:
          return AppResponse.named(
            error: "Cancelled by user",
          );
      }
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }
}
